@@ -729,6 +729,10 @@ pref("plugin.disable", false);
 
 // Digital Rights Management, Encrypted Media Extensions
 pref("media.eme.enabled", false);
+ 
+// Turn off WebRTC by default (bug 1419507)
+pref("media.navigator.enabled", false);
+pref("media.peerconnection.enabled", false);
 
 #ifndef XP_MACOSX
 // Restore the spinner that was removed in bug 481359
